PRO read_trueimage, FILE, image, r,g,b, DITHER=DITHER, CURRENT=CURRENT, $
      eightbit=eightbit 

;+
; NAME:
;    read_trueimage
;
; PURPOSE:
;    Show the contents of a graphics file in the current window.
;
;    The input formats supported are:
;    BMP   8-bit with color table or 24-bit true-color,
;    TIFF  8-bit with color table or 24-bit true-color,
;    JPEG 24-bit true color,
;
;    Any conversions necessary to translate 8-bit or 24-bit files
;    to 8-bit or 24-bit images on-screen
;
; CATEGORY:
;    Input/Output.
;
; CALLING SEQUENCE:
;    read_trueimage,FILE, image, r,g,b
;
; INPUTS:
;    FILE     Name of the output file (format is identified automatically).
;
; OPTIONAL INPUTS:
;    None.
;
; KEYWORD PARAMETERS:
;    DITHER   Set this keyword to dither the input image when displaying
;             24-bit images on an 8-bit display (default is no dithering).
;    CURRENT  Set this keyword to display the image in the current window
;             (default is to create a new window sized to fit the image).
;    eightbit - if set, convert 24->8bit
;
; OUTPUTS:
;    None.
;
; OPTIONAL OUTPUTS:
;    None
;
; COMMON BLOCKS:
;    None
;
; SIDE EFFECTS:
;    The color table is modified.
;
; RESTRICTIONS:
;    Requires IDL 5.2 or higher (image QUERY functions).
;
; EXAMPLE:
;
;
; MODIFICATION HISTORY:
; Liam.Gumley@ssec.wisc.edu
; http://cimss.ssec.wisc.edu/~gumley
; $Id: showimage.pro,v 1.15 1999/11/19 21:20:50 gumley Exp $
; 9-October-2001 - used the read/convert logic (removed display)
;
; Copyright (C) 1999 Liam E. Gumley
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;-

rcs_id = '$Id: showimage.pro,v 1.15 1999/11/19 21:20:50 gumley Exp $'

;-------------------------------------------------------------------------------
;- CHECK INPUT
;-------------------------------------------------------------------------------

;- Check IDL version

if float(!version.release) lt 5.2 then begin
  message, 'IDL 5.2 or higher is required', /continue
  return
endif

;- Check input arguments

case 1 of 
   n_params() lt 2: begin
      box_message,'IDL> read_trueimage,file,image[r,g,b]'
      return
   endcase
   1-file_exist(file): begin 
      box_message,['Need to supply filename', $
         'IDL> read_trueimage,file,image[r,g,b]']
      return
   endcase
   else:
endcase

;-------------------------------------------------------------------------------
;- CHECK THE GRAPHICS DEVICE
;-------------------------------------------------------------------------------

; (removed - S.L.Freeland , 9-oct-2001 - see showimage.pro )
;-------------------------------------------------------------------------------
;- IDENTIFY FILE AND READ IMAGE
;-------------------------------------------------------------------------------

;- Identify the file format

result = query_gif(file, info)
if result eq 0 then result = query_bmp(file, info)
if result eq 0 then result = query_pict(file, info)
if result eq 0 then result = query_tiff(file, info)
if result eq 0 then result = query_jpeg(file, info)
if result eq 0 then begin
  message, 'File format not recognized', /continue
  return
endif

;- Fix the channel information for GIF images

if info.type eq 'GIF' then info.channels = 1

;- Read the image

case info.type of

  'GIF' : read_gif, file, image, r, g, b

  'BMP' : begin
    if info.channels eq 1 then begin
      image = read_bmp(file, r, g, b)
    endif else begin
      image = read_bmp(file)
      image = reverse(temporary(image), 1)
    endelse
  end

  'PICT' : read_pict, file, image, r, g, b

  'TIFF' : begin
    if info.channels eq 1 then begin
      image = read_tiff(file, r, g, b, order=order)
      image = reverse(temporary(image), 2)
    endif else begin
      image = read_tiff(file, order=order)
      image = reverse(temporary(image), 3)
    endelse
  end

  'JPEG' : read_jpeg, file, image

endcase

;- If an 8-bit image was read, reduce the number of colors
if info.channels eq 1 then begin
  dtemp=!d.name
  set_plot,'z'
  reduce_colors, image, index
  r = r[index]
  g = g[index]
  b = b[index]
  set_plot,dtemp
endif

;- Get image size

dims = size(image, /dimensions)
if n_elements(dims) eq 2 then begin
  nx = dims[0]
  ny = dims[1]
endif else begin
  nx = dims[1]
  ny = dims[2]
endelse

;-------------------------------------------------------------------------------
;- CREATE A WINDOW
;-------------------------------------------------------------------------------
; (removed S.L.Freeland - 9-Oct-2001 ; see showimage.pro )
;-------------------------------------------------------------------------------
;- HANDLE IDL 8-BIT MODE
;-------------------------------------------------------------------------------

eightbit=keyword_set(eightbit)

if eightbit then begin 

  ;- If the color table of an 8-bit image is larger than
  ;- the current display table, convert the image to 24-bit

  if (info.channels eq 1) and (n_elements(r) gt !d.table_size) then begin

    ;- Convert to 24-bit

    dims = size(image, /dimensions)
    nx = dims[0]
    ny = dims[1]
    true = bytarr(3, nx, ny)
    true[0, *, *] = r[image]
    true[1, *, *] = g[image]
    true[2, *, *] = b[image]
    image = temporary(true)

    ;- Reset the number of channels

    info.channels = 3

  endif

  ;- If image is 24-bit, convert to 8-bit

  if info.channels eq 3 then begin

    ;- Convert 24-bit image to 8-bit

    image = color_quan(image, 1, r, g, b, colors=255, $
      dither=keyword_set(dither))

    ;- Sort the color table from darkest to brightest

    table_sum = total([[long(r)], [long(g)], [long(b)]], 2)
    table_index = sort(table_sum)
    image_index = sort(table_index)
    r = r[table_index]
    g = g[table_index]
    b = b[table_index]
    oldimage = image
    image[*] = image_index[temporary(oldimage)]

    ;- Reset the number of channels

    info.channels = 1

  endif

endif

;-------------------------------------------------------------------------------
;- DISPLAY THE IMAGE
;-------------------------------------------------------------------------------
; (removed S.L.Freeland - 9-Oct-2001; see showimage.pro for display routine) 

END
