function ssw_gevloc_summovie, gevloc, difference=difference, $
   debug=debug, _extra=_extra, $
   lastn_hours=lastn_hours, above_class=above_class, goes=goes , $
   xsize=xsize, fov=fov, index=fovii,$
   plot_goes_snap=plot_goes_snap, ascii=ascii
;
;+
;   Name: ssw_gevloc_summovie
;
;   Purpose: generate summary visualization of 'latest_events' dbase
;
;   Input Paramters:
;      gevloc - optional vector of desired 'gevloc' dbase records
;               (default is current contents of 'latest_events)
;
;   Output Parameters:
;      Function output is 3D cube which includes one context image per gev rec
;
;   Keyword Parameters:
;      difference - if set, output images are difference images
;      lastn_hours - only includes events in last NN hours
;      above_class - only include events >= this GOES class
;      goes (switch) - if set, include annotated GOES sxr Light Curve  
;      _extra - other keywords passed to ssw_fov_context_cube.pro
;      FOV - optional field of view in arcmin (default=40)
;      XSIZE - optional output framesize in X; default=512
;      
;   Calling Sequence:
;      IDL> sumcube=ssw_gevloc_summovie([gevloc] [,lastn_hours=NN] $
;                                                 [,above_class=CLASS] $
;                                                 [,/GOES][,_EXTRA=_EXTRA] )
;

;   History:
;      Circa 1-May-2003 - S.L.Freeland - played with this for a few hours
;      16-Sep-2003      - S.L.Freeland - dusted off, made it work, added
;                                        LASTN_HOURS, ABOVE_CLASS, GOES
;                                        keyword and functions.
;      21-Oct-2003 - S.L.Freeland - add /GOES_SNAPSHOT keyword and function
;
;   Restrictions:
;      1. If not local to 'latest_events' (LMSAL) then assumes
;         IDL Version >=5.4 (calls get_gevloc_data which uses RSI 'socket')
;      2. Requires EIT recent Level 1 data queue which is available
;            via $SSWDB/soho/eit/eit_fulldisk_l1q      
;
;   Method:
;      using gevloc info, setup and call 'ssw_fov_context_cube.pro'
  
common ssw_gevloc_summovie_blk,dbok  ; only check/recheck if Never checked or found

if n_elements(fov) eq 0 then fov=40       ; default Field of View=40 ArcMin
if n_elements(xsize) eq 0 then xsize=512  ; default image size

if n_elements(dbok) eq 0 then dbok=0
if not dbok then begin 
   l1qdir=(concat_dir('$SSWDB','soho/eit/eit_fulldisk_l1q'))(0)
   dirok=file_exist(l1qdir)
   if dirok then dbok=n_elements(findfile(l1qdir)) gt 1
   if not dirok then begin 
      box_message,'Sorry, you need EIT recent Level 1 Queue > ' + $
         '$SSWDB/soho/eit/eit_fulldisk_l1q'
      return,-1
   endif
endif
if not data_chk(gevloc,/struct) then  gevloc=get_gevloc_data() 

ssok=where(gevloc.helio ne '',okcnt)
if okcnt eq 0 then begin 
   box_message,'No valid events...'
   return,-1
endif

gevloc=gevloc(ssok)

; optionally limit by most recent NN hours...
if keyword_set(lastn_hours) then begin 
   sst=sel_timrange(anytim(gevloc.date_obs,/int),$       
      reltime(hours=-1*abs(lastn_hours)), reltime(/now),/between)
   if sst(0) eq -1 then begin 
      box_message,'No events in last '+strtrim(lastn_hours,2) + ' hours...'
      return,-1
   endif
   gevloc=gevloc(sst)
endif

; optionally filter on minimum GOES class
if keyword_set(above_class) then begin 
   gcut=strupcase(strtrim(above_class,2))
   ssc=where(gevloc.class ge gcut,gcnt)
   if gcnt eq 0 then begin 
      box_message,'No events >= ' + gcut
      return,-1
   endif
   gevloc=gevloc(ssc)
endif
nev=n_elements(gevloc)
efiles=str2cols(gevloc.lfiles,',',/un)

strtab2vect,efiles,f0,f1

l1q=sswdb_files(/l1q)                 ; all eit l1q (summovie movie dbase)
etimes=file2time(efiles,out='int')    ; times
elot=file2time(f0,out='int')          ; times of locator image background
ehit=file2time(f1,out='int')          ; times of locator image (peak or rise)

l1qt=file2time(l1q,out='int')         ; times of l1q

ss0=tim2dset(l1qt,elot)               ; closest movie dbase to loc0 (bkgd)
ss1=tim2dset(l1qt,ehit)               ; closest movie dbase to loc1 (peak)
loc0=l1q(ss0)
loc1=l1q(ss1)

fok=where(file_exist(loc0) and file_exist(loc1),okcnt) ; ~redundant 

if okcnt eq 0 then begin 
   box_message,'No local flare location files(?) in ' + pp(0)
   return,-1
endif

loc0=loc0(fok)
loc1=loc1(fok)
gevloc=gevloc(fok)

mreadfits,loc0,i0         ; headers only 
mreadfits,loc1,i1         ; headers only 
ssw_fake_index,gevloc.date_obs,ii,dd,nx=100,xcen=gevloc.xcen,$
 ycen=gevloc.ycen,cdelt1=i1(0).cdelt1

if keyword_set(difference) then begin 
   mreadfits,loc0(0),index,data
   odiff=make_array(i0(0).naxis1,i0(0).naxis2,nev,/nozero,/float)
   for i=0,nev-1 do begin 
      mreadfits,loc0(i),ii0,dd0
      mreadfits,loc1(i),ii1,dd1
      odiff(0,0,i)=float(temporary(dd1))-float(temporary(dd0))
   endfor
stop,'difference,odiff'
 
endif else begin 
   mreadfits,loc1,i1,d1
   d1=d1>0
   ssw_fill_cube,d1
   ;sdata=do_eit_scaling(d1,/log,/no_prep)
   sdata=ssw_sigscale(i1,d1,/corner,/log)
endelse

index2map,ii,dd,ref_map
index2map,i1,sdata,emaps
nmaps=n_elements(emaps)

sub_map,emaps(0),fovmap,ref_map=ref_map(0)
fovmaps=replicate(fovmap,nmaps)
for i=1,nmaps-1 do begin 
  sub_map,emaps(i),fovmap,ref_map=ref_map(i) 
  fovmaps(i)=str_copy_tags(fovmaps(i),fovmap)
endfor
map2index,add_tag(fovmaps,0.,'roll'),fovii,fovdd

fovii.xcen=gevloc.xcen
fovii.ycen=gevloc.ycen
fovii.crpix1=0
fovii.crpix2=0
fovii=struct2ssw(fovii)

fddata=ssw_fov_context_cube(fovii,fovdd,margin=.05,/oplotnar, $
   fdmap=emaps,xsize=xsize,_extra=_extra,center=[0,0],fov=40)

if keyword_set(goes) then begin 
   event_movie, fovii, fddata, data=temporary(fddata),  $
      /goes, /reverse, ascii=ascii, twindow=120
endif

if keyword_set(plot_goes_snap) or get_logenv('plot_goes_snap') ne '' then begin 
   ptitle='!6GOES ' + gevloc.class + ' ' + gevloc.helio + '!C'+ gevloc.ename
   !p.color=0
   !color=0
   goes_cube=plot_goes_cube(gevloc.fstart, gevloc.fstop, $
      xsize=xsize/4,charsize=.5,xmargin=[5,0],ymargin=[0,4],$
      gcolor=100,evt_times=gevloc.fpeak,/arrow,ascii=ascii,$
      title=ptitle, background=250,color=2, window_minute=30)

   fddata(xsize-(xsize/4),0,0)=goes_cube
endif

return,fddata
end

