pro index2fov, index, east, west, north, south, $
	       extremes=extremes, heliographic=heliographic, $
	       center_fov=center_fov, size_fov=size_fov, $ 
               l1=l1, soho=soho
;+
;   Name: index2fov
;
;   Purpose: FOVs for input index(s) - optionally EXTREMEs of multiple FOVs
;  
;   Input Parameters:
;      index - vector of SSW structures (inc pointing and time standards..)
;
;   Output Parameters:
;      east, west, north, south - image edges in arcseconds
;
;   Keyword Parameters:
;      extremes - (input) - switch, if set, outputs are for "composite"
;      heliographic - input - switch, if set, outputs are heliographic
;      l1/soho - (input) switches - synonym for L1 in arcmin2hel call
;                             (default=heliocentric in arcsec)
;      center_fov - (output) - if /extremes, then "composite" fov center
;      size_fov   - (output) - if /extremes, then "composite" fov nx,ny
;        
;-

if not required_tags(index,'naxis1,xcen,cdelt1',missing=missing ) then begin
   box_message,['The following required tags are missing', '   ' + missing, $
		'IDL> index2fov,index, east,west,north,south']
   return
endif   


; ---------- extract the required info ---------
xcen=gt_tagval(index,/xcen)
ycen=gt_tagval(index,/ycen)
cdelt1=gt_tagval(index,/cdelt1)
cdelt2=gt_tagval(index,/cdelt2,missing=cdelt1)
naxis1=gt_tagval(index,/naxis1)
naxis2=gt_tagval(index,/naxis2)

; ---------- calc fov in " from sun center ---------
dx=float(naxis1*cdelt1)/2.
dy=float(naxis2*cdelt2)/2.
east=xcen-dx
west=xcen+dx                                ; WEST Positive
north=ycen+dy                               ; NORTH Positive
south=ycen-dy

if keyword_set(extremes) then begin         ; limits for multiple FOVs
   east=min(east)                           
   west=max(west)
   north=max(north)
   south=min(south)
   size_fov=[west-east, north-south]             ; NX,NY
   center_fov=([west,north]) - float(size_fov)/2.
endif
stop
if keyword_set(heliographic) then begin 
   l1=keyword_set(l1) or keyword_set(soho) or $
      (strupcase(gt_tagval(index(0),/TELESCOP)) eq 'SOHO')
   utcints=anytim(index,/utc_int)

   helul=arcmin2hel(east/60.,north/60.,soho=l1,date=utcints)
   helll=arcmin2hel(east/60.,south/60.,soho=l1,date=utcints)
   helur=arcmin2hel(west/60.,north/60.,soho=l1,date=utcints)
   hellr=arcmin2hel(west/60.,south/60.,soho=l1,date=utcints)
   east=helul(0,*) < helll(0,*)
   west=helur(0,*) > hellr(0,*)
   north=helul(1,*) > helur(1,*)
   south=helll(1,*) < hellr(1,*)
endif

return
end
