pro ssw_index2pfss, sswindex, pfssindex, pfssdata , $
   open_color=open_color, closed_color=closed_color, $
   refresh=refresh, force_remote=force_remote,   $
   strref=strref, fieldtype=fieldtype, $
   image_reference=image_reference, $
   nlines=nlines, bbox=bbox, debug=debug, $ 
   lcent=lcent, bcent=bcent,  earth_view=earth_view, $
   _extra=_extra, before=before, after=after, $
   ecliptic=ecliptic, spacing=spacing, chboundries=chboundries, $
   draw_field2=draw_field2, crop=crop
;+
;   Name: ssw_index2pfss
;
;   Purpose: interface from SSW 'index' or 'map' to PFSS derived field lines.
;
;   Input Parameters:
;      sswindex - an ssw 'index' record or Map per D.M.Zarro et al
;
;   Output Parameters:
;     pfssindex - sswindex and pfss parameter derived composite index
;     pfssdata - the 2D field line  projection  
;
;   Keyword Parameters:
;      open_color - if set, value to set OPEN lines
;      closed_color - if set, value to set CLOSED lines
;      _extra - Undefined keywords -> pfss_draw_field via inheritance
;               (MAG, WIDTH , crop, imsc, thick, drawopen, drawclosed...)
;     fieldtype - per pfss_field_start_coord, namely:
;;                           1=points fall along a line
;                            2=points are distributed uniformly in latitude 
;                              and randomly in longitude
;                            3=points are distributed randomly in both
;                              latitude and longitude (default)
;                            4=read in from a file
;                            5=uniform grid 
;                            6=points are weighted by flux
;
;     before/after - optional switches to force Bfield dbase selection
;                    (closest preceding, closest after) - def is closest
;     strref - desired start point for field; units=radius 
;              where 1.0=photosphere and 2.5 = source surface 
;     bbox - optional subfield for field line restriction tie points.
;            May be specifed as:
;               a) 4 element vector (per pfss_field_start_coord)
;               b) SSW 'index' reference record (FITs header derived for example
;               c) 'map' per DMZarro et al.
;
;     ecliptic (switch) - if set, limit lines to ecliptic open lines
;     chboundries (switch) - if set, include Coronal Hole boundries
;     draw_field2 (switch) - if set, use draw_field2 instead of draw_field
;
;   Restrictions:
;      /CHBOUNDRIES currenly only works when /ECLIPTIC set 
;
;   Notes: also see ssw_index2pfss_multi, a driver for This program which calls
;          and accumulates lines for a vector of ref
;     
;   History:
;      5-December-2003 - S.L.Freeland - command line interface ssw->pfss
;                                       suite by M.Derosa et al. 
;     20-Jan-2004      - S.L.Freeland - got remote access working
;                                       (via pfss_time2file/pfss_restore)
;      2-Feb-2004 - S.L.Freeland - tracing/rendering pieces 
;     16-Mar-2004 - S.L.Freeland - add /EARTH_VIEW
;     12-apr-2004 - S.L.Freeland - add FIELDTYPE and STRREF keyword+function
;      3-jan-2006 - S.L.Freeland - add /ECLIPTIC and /CHBOUNDRIES keywords
;                                  based on recent mods/script from Marc DeRosa
;      7-Jan-2006 - S.L.Freeland - add /DRAW_FIELD2 keyword&function
;     10-Jan-2006 - S.L.Freeland - call pfss_fake_index to rationalize output 'index'
;;
@pfss_data_block

debug=keyword_set(debug)
earth_view=keyword_set(earth_view)

if n_params() lt 2 then begin 
   box_message,'IDL> ssw_index2pfss,sswindex,pfssindex,pfssdata [,options]'
   return
endif 

pfssf=pfss_time2file(sswindex,/ssw_cat,after=after,before=before)
if pfssf(0) eq '' then begin 
   box_message,'Problem mapping time->pfss field file name'
   return
endif

if not file_exist(pfssf) or keyword_set(force_remote) then begin 
   pfssf=pfss_time2file(sswindex,/ssw_cat,/url)   ; try remote access
endif

pfss_restore,pfssf, refresh=refresh

if not keyword_set(nlines) then nlines=100

if keyword_set(earth_view) then begin 
   lcent=l0 
   bcent=b0
endif

ptemp=!d.name
set_plot,'z'
case 1 of 
   earth_view and n_elements(image_reference) eq 0: $
      bboxx=[((l0-90)+360) mod 360, -90, ((l0+90)+360) mod 360,90]
   n_elements(bbox) eq 0 and n_elements(image_reference) eq 0: $
      bboxx=[280,-90,80,90]                ; no clue, use full sun
   n_elements(bbox) eq 4: bboxx=bbox       ; user supplied bbox
   else: begin 
      case 1 of 
         valid_map(bbox): map2index, bbox, index ; DMZ map -> index
         required_tags(bbox,'crpix1,cdelt1,naxis1'): index=bbox
         required_tags(image_reference,'crpix1,cdelt1,naxis1'): $
             index=image_reference
         else:
      endcase 
      index2fov,index,E,W,N,S,/helio
      E=([E,(E+360) mod 360])(E lt 0)
      W=([W,(W+360) mod 360])(W lt 0)
      bboxx=[E,S,W,N]
   endcase
endcase 
print,bboxx
draw_field2=keyword_set(draw_field2)
draw_func='pfss_draw_field'+(['','2'])(draw_field2)
if keyword_set(ecliptic) then begin ; special processing, at least for now...
   idate=anytim(sswindex,/ecs)
   lcent=(tim2carr(idate))(0) ; carrington long in degrees
   pb0rdata=pb0r(idate)
   bcent=pb0rdata(1) ; b angle in degrees
   if n_elements(necliptic) eq 0 then necliptic=120 ; M.DeRosa default
   pfss_field_start_coord,1,necliptic,radstart=2.5
   if n_elements(spacing) eq 0 then spacing=2.5    ; M.Derosa default
   pfss_field_start_coord,5,spacing,radstart=2.5,/add
   pfss_field_start_coord,15,spacing,radstart=rix(1),/add
   pfss_trace_field

   ;  get the coronal hole boundaries
   pfss_get_chfootprint,openfield2,/quiet,/usecurrent  ;  for debugging
   pfss_get_chfootprint,openfield,/quiet,/usecurrent,/close,spacing=spacing
;  rotate open field grid so that lcent is in the middle of the image (do this
;  because the contouring gives imperfections at the edges, so hide these on
;  the non-earth-facing side of the sun)
   wh=where(lon gt lcent,nwh)
   if nwh gt 0 then openoffset=(wh(0)+nlon/2) mod nlon else openoffset=nlon/2
   openfield=shift(openfield,-openoffset)

;  smooth image slightly so that the contours are not as angular
   openfield=filter_image(openfield,fwhm=1.5)  ;  in IDL astron library

;  plot the disk image
   call_procedure,draw_func,imsc=200,outim=imdisk,lcent=lcent,bcent=bcent, $
     /nolines,_extra=_extra

;  get [x1,y1,x2,y2] of orthographic projection of disk
   naxdisk=size(imdisk,/dim)
   case 1 of 
      data_chk(_extra,/struct): width=gt_tagval(_extra,/width,missing=2.5)
      else: width=2.5
   endcase
   diam=float(naxdisk)/width
   x1=round((naxdisk(0)-diam(0))/2)  &  x2=round(x1+diam(0))
   y1=round((naxdisk(1)-diam(1))/2)  &  y2=round(y1+diam(1))
   openpos=float([x1,y1,x2,y2])/[naxdisk,naxdisk]                      

;  draw coronal hole contours on the orthographic projection
   device,set_resolution=naxdisk
   scim,imdisk
   map_set,/ortho,pos=openpos,/noer,/nobor,bcent,lcent-lon(openoffset)
   contour,openfield,lon,lat,/overplot,/follow, $
     levels=[-0.03,0.03],c_labels=[0,0],c_colors=[249,0],pos=openpos,/norm
   imdisk2=tvrd()
;  get fieldilne image
   str=str(0:necliptic-1)
   stth=stth(0:necliptic-1)
   stph=stph(0:necliptic-1)
   nstep=nstep(0:necliptic-1)
   call_procedure,draw_func,imsc=200,outim=imlines,lcent=lcent,bcent=bcent, $
   _extra=_extra,/noimage,/quiet

;  combine images
   outim=temporary(imdisk2)
   wh=where(imlines gt 0,nwh)
   if nwh gt 0 then outim(wh)=imlines(wh)
endif else begin 
   if n_elements(fieldtype) eq 0 then fieldtype=3 ; random
   pfss_field_start_coord, fieldtype, nlines, bbox=bboxx 
   if n_elements(strref) eq 0 then strref=rix(1)  ; ??
   str(*)=strref
   if debug then stop
   pfss_trace_field 
   call_procedure,draw_func,imsc=200,outim=outim,_extra=_extra, lcent=lcent, bcent=bcent, crop=crop
endelse

pfssindex=pfss_fake_index(sswindex,_extra=_extra)
pfssdata=temporary(outim)

set_plot,ptemp

if debug then stop


return
end



